@@ -50,6 +50,7 @@ G_BEGIN_DECLS
 typedef struct {
   GObject parent;
   GKeyFile *keyfile;
+  GKeyFile *secrets;
   GKeyFile *removed;
   GHashTable *removed_accounts;
   gchar *filename;
@@ -70,6 +71,4 @@ GType mcd_account_manager_default_get_type (void) G_GNUC_CONST;
 
 McdAccountManagerDefault *mcd_account_manager_default_new (void);
 
-G_END_DECLS
-
 #endif
