@@ -2,7 +2,7 @@
  * deliberately a "smaller" API than McdAccountManager.
  *
  * Copyright © 2010 Nokia Corporation
- * Copyright © 2010-2012 Collabora Ltd.
+ * Copyright © 2010 Collabora Ltd.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -25,20 +25,14 @@
 #include "plugin-account.h"
 #include "config.h"
 
-#include <string.h>
-
-#include <telepathy-glib/telepathy-glib.h>
-
 #include "mission-control-plugins/implementation.h"
 
+#include <telepathy-glib/util.h>
+
 /* these pseudo-plugins take care of the actual account storage/retrieval */
 #include "mcd-account-manager-default.h"
-
 #if ENABLE_LIBACCOUNTS_SSO
 #include "mcd-account-manager-sso.h"
-# ifdef ACCOUNTS_GLIB_HIDDEN_SERVICE_TYPE
-# include "mcd-storage-ag-hidden.h"
-# endif
 #endif
 
 static GList *stores = NULL;
@@ -234,7 +228,7 @@ unique_name (const McpAccountManager *ma,
   const gchar *base = NULL;
   gchar *esc_manager, *esc_protocol, *esc_base;
   guint i;
-  gsize base_len = strlen (TP_ACCOUNT_OBJECT_PATH_BASE);
+  gsize base_len = sizeof (MC_ACCOUNT_DBUS_OBJECT_BASE) - 1;
   DBusGConnection *connection = tp_proxy_get_dbus_connection (self->dbusd);
 
   base = tp_asv_get_string (params, "account");
@@ -248,8 +242,8 @@ unique_name (const McpAccountManager *ma,
 
   for (i = 0; i < G_MAXUINT; i++)
     {
-      gchar *path = g_strdup_printf (
-          TP_ACCOUNT_OBJECT_PATH_BASE "%s/%s/%s%u",
+      gchar *path = g_strdup_printf ("%s%s/%s/%s%u",
+          MC_ACCOUNT_DBUS_OBJECT_BASE,
           esc_manager, esc_protocol, esc_base, i);
 
       if (!g_key_file_has_group (self->keyfile, path + base_len) &&
@@ -281,19 +275,12 @@ account_storage_cmp (gconstpointer a, gconstpointer b)
 }
 
 static void
-add_storage_plugin (McpAccountStorage *plugin)
-{
-  stores = g_list_insert_sorted (stores, plugin, account_storage_cmp);
-}
-
-static void
-add_libaccounts_plugins_if_enabled (void)
+add_libaccount_plugin_if_enabled (void)
 {
 #if ENABLE_LIBACCOUNTS_SSO
-  add_storage_plugin (MCP_ACCOUNT_STORAGE (mcd_account_manager_sso_new ()));
-# ifdef ACCOUNTS_GLIB_HIDDEN_SERVICE_TYPE
-  add_storage_plugin (MCP_ACCOUNT_STORAGE (mcd_storage_ag_hidden_new ()));
-# endif
+    McdAccountManagerSso *sso_plugin = mcd_account_manager_sso_new ();
+
+    stores = g_list_insert_sorted (stores, sso_plugin, account_storage_cmp);
 #endif
 }
 
@@ -301,17 +288,18 @@ static void
 sort_and_cache_plugins ()
 {
   const GList *p;
+  McdAccountManagerDefault *default_plugin = NULL;
   static gboolean plugins_cached = FALSE;
 
-  if (plugins_cached)
-    return;
-
   /* not guaranteed to have been called, but idempotent: */
   _mcd_plugin_loader_init ();
 
-  /* Add compiled-in plugins */
-  add_storage_plugin (MCP_ACCOUNT_STORAGE (mcd_account_manager_default_new ()));
-  add_libaccounts_plugins_if_enabled ();
+  /* insert the default storage plugin into the sorted plugin list */
+  default_plugin = mcd_account_manager_default_new ();
+  stores = g_list_insert_sorted (stores, default_plugin, account_storage_cmp);
+
+  /* now poke the pseudo-plugins into the sorted GList of storage plugins */
+  add_libaccount_plugin_if_enabled ();
 
   for (p = mcp_list_objects(); p != NULL; p = g_list_next (p))
     {
@@ -319,7 +307,7 @@ sort_and_cache_plugins ()
         {
           McpAccountStorage *plugin = g_object_ref (p->data);
 
-          add_storage_plugin (plugin);
+          stores = g_list_insert_sorted (stores, plugin, account_storage_cmp);
         }
     }
 
@@ -554,38 +542,6 @@ _storage_dup_value (McdStorage *storage,
                 value = tp_g_value_slice_new_take_object_path (v_string);
               }
           }
-        else if (type == TP_ARRAY_TYPE_OBJECT_PATH_LIST)
-          {
-            gchar **v =
-              g_key_file_get_string_list (keyfile, account, key, NULL, error);
-            gchar **iter;
-            GPtrArray *arr = g_ptr_array_new ();
-
-            for (iter = v; iter != NULL && *iter != NULL; iter++)
-              {
-                if (!g_variant_is_object_path (*iter))
-                  {
-                    g_set_error (error, MCD_ACCOUNT_ERROR,
-                        MCD_ACCOUNT_ERROR_GET_PARAMETER,
-                        "Invalid object path %s stored in account", *iter);
-                    g_strfreev (v);
-                    v = NULL;
-                    break;
-                  }
-              }
-
-            for (iter = v; iter != NULL && *iter != NULL; iter++)
-              {
-                /* transfer ownership from v to arr */
-                g_ptr_array_add (arr, *iter);
-              }
-
-            /* not g_strfreev - the strings' ownership has been transferred */
-            g_free (v);
-
-            value = tp_g_value_slice_new_take_boxed (
-              TP_ARRAY_TYPE_OBJECT_PATH_LIST, arr);
-          }
         else
           {
             gchar *message =
@@ -666,8 +622,6 @@ update_storage (McdPluginAccountManager *self,
               pn, done ? "store" : "ignore", account, key);
         }
     }
-
-  g_free (val);
 }
 
 static gboolean
@@ -777,13 +731,6 @@ _storage_set_value (McdStorage *storage,
                 g_key_file_set_string (self->keyfile, name, key,
                     g_value_get_boxed (value));
               }
-            else if (G_VALUE_HOLDS (value, TP_ARRAY_TYPE_OBJECT_PATH_LIST))
-              {
-                GPtrArray *arr = g_value_get_boxed (value);
-
-                g_key_file_set_string_list (self->keyfile, name, key,
-                    (const gchar * const *) arr->pdata, arr->len);
-              }
             else
               {
                 g_warning ("Unexpected param type %s",
@@ -810,7 +757,6 @@ _storage_set_value (McdStorage *storage,
           updated = TRUE;
         }
 
-      g_free (new);
       g_free (buf);
       g_free (old);
 
